unit UnitDFT;

interface

uses Classes;
const
    //  4     discret = 20000;  SizeBufOut = 400; DFT_kmax=200/100;
    //      20   
    discret = 16000;  // per second
    SizeBufOut = 640; //     
    SizeBufIn = SizeBufOut;   //   SizeBufIn/discret = 40 msec
    DFT_nmax=SizeBufIn;
    DFT_kmax=256; //  


type
    DFT_Koef= array[0..DFT_nmax-1,0..DFT_kmax-1] of single;
    DFT_spectr = array[0..DFT_kmax-1] of smallint;
    TScart = array of DFT_spectr; //      


type
 TImageSound = class
  private
  public
   Data:TScart;
   MaxHarm:TScart;
   Start_dim: array of smallint;
   Stop_dim: array of smallint;
   Power:array of smallint;
   AvgHarm:array of single;//Envelope.Count_frame
   AvgFrame:array of single;//DFT_kmax
   DataUp,DataRight,DataDown:TScart;
   avg: array[0..3] of single;
   procedure CreateDataNext;

 end;


 TEnvelope = class
  private
  public
    Time:integer; //     ms
    LevelAmpl:integer;
    LevelPower:integer;
    LevelDif:integer;
    LevelMax:integer;
    Frame_ms:integer;//    ms
    Count_frame:integer; //    Envelope.Time
    tick:integer;//  
    Ampl:integer; // trigger
    Start_dim: array of smallint;
    Stop_dim: array of smallint;
    flipstart,flipstop:boolean;
    min_X,max_X:integer;
    hifiltr:DFT_spectr;
    K_expand:integer;
    K_HiF:integer;
    Gain:integer;

    CountHarmFrame: array of smallint;


  constructor Create;
  procedure Find_maxHarm(data:TScart; var mH:TScart);
 end;

 TRecognize = class
  private
  public
    Common_coincident: byte;
    procedure Compare(dat,pat:TImageSound; var pr:integer);
 end;

 TDFT = class
  private
  public
    nmax:integer;
    kmax:integer;
    T:real;
    K_sin,K_cos:DFT_Koef;
    fdata: array of integer;
    Win_Hamming,Win_Blackman,Win_BlackmanNuttall: array of real;

    constructor Create;
    Function ReDFTQ (kg:integer):real;
    Function ImDFTQ (kg:integer):real;
    Procedure Create_SinCos;
  end;

implementation

{*************************************************}
constructor TDFT.Create;
begin
  SetLength(fdata, SizeBufIn);
  kmax:=DFT_kmax; //  
  nmax:= DFT_nmax;
end;

{*************************************************}
procedure TDFT.Create_SinCos;
var k,n:word;

begin
 T := 2*pi/nmax;
 for k:=0 to kmax-1 do
  for n:=0 to nmax-1 do
   begin
     K_sin[n,k]:=sin(n*k*T);
     K_cos[n,k]:=cos(n*k*T);
   end;
end;
{*************************************************}

Function TDFT.ReDFTQ (kg:integer):real;
var
 sum:single;
 n:word;
begin
 sum:=0;
 for n:=0 to nmax-1 do  sum:=sum+fdata[n]*K_cos[n,kg];
// for n:=0 to (nmax div 4)-1 do  sum:=sum+fdata[n]*K_cos[n,kg];
 ReDFTQ:=sum;
end;
{*************************************************}
Function TDFT.ImDFTQ (kg:integer):real;
var
 sum:single;
 n:word;
begin
 for n:=0 to nmax-1 do sum:=sum+fdata[n]*K_sin[n,kg];
// for n:=0 to (nmax div 4)-1 do sum:=sum+fdata[n]*K_sin[n,kg];
 ImDFTQ:=sum;
end;


constructor TEnvelope.Create;
var k,km:integer;
begin
    Time:=2000; //     ms,  
    LevelAmpl:=0; //       ,  
    LevelPower:=1200;//     ,   
    //     -   Start_dim  Stop_dim
    LevelMax:=5;//,      
    //   , ...
    tick:=0;
    Frame_ms:=1000*SizeBufIn div discret; //  20ms
    Count_frame:=Time div Frame_ms;
    Ampl:=10; //  

    SetLength(Start_dim,Count_frame);
    SetLength(Stop_dim,Count_frame);
    SetLength(CountHarmFrame,Count_frame);
    K_expand:=40;

{
    K_HiF:=40;
    for k:=0 to DFT_kmax-1 do
     begin
      if k> K_HiF then km:=400 else km:=k;
      hifiltr[k]:=1;
     end;
}
    LevelDif:=10; //      
end;

procedure TEnvelope.Find_maxHarm(data:TScart; var mH:TScart);
var
 n,k:integer;
 old_harm:integer;
 diff:integer;
 fazaup:boolean;
 Ai:integer;
begin

 SetLength(mH,Count_frame);
 for n:=0 to Count_frame - 1 do
 for k:=0 to DFT_kmax-1 do
 begin
    Ai:= data[n][k];
    if k=0 then
    begin
      mH[n][k]:=0;
      old_harm:=Ai;
      fazaup:=true;
    end else
    begin
      diff:= old_harm-Ai;
      if Ai > LevelAmpl then
       if diff > LevelDif then
       begin
         if fazaup then
         begin
           MH[n][k-1]:=1;
           fazaup:=false;
         end
           else MH[n][k-1]:=0;
       end
        else
       begin
         MH[n][k-1]:=0;
         fazaup:=true;
       end;
       old_harm:=Ai;
    end;
/////////////////////////
 end; //k




end;



procedure TRecognize.Compare(dat,pat:TImageSound; var pr:integer);
var k,n,np:integer;
 start_posX_dat:integer;
 stop_posX_dat:integer;
 start_posX_pat:integer;
 stop_posX_pat:integer;

 max_harm_dat:integer;
 max_harm_pat:integer;
 old_max_harm:integer;

 frames:integer;
 ddd,ppp:integer;
 sum_frame:integer;
 sum_all:integer;
begin

 start_posX_dat:=0;
 stop_posX_dat:=0;
 start_posX_pat:=0;
 stop_posX_pat:=0;


 frames:=Length(dat.Start_dim);
 for n:=0 to frames-1 do //         X
 begin
  if (dat.Start_dim[n] > 0) and (start_posX_dat = 0)  then start_posX_dat:=n;
  if (pat.Start_dim[n] > 0) and (start_posX_pat = 0)  then start_posX_pat:=n;
  if (dat.Stop_dim[n] > 0) and (stop_posX_dat < frames) then stop_posX_dat:= n;
  if (pat.Stop_dim[n] > 0) and (stop_posX_pat < frames) then stop_posX_pat:= n;
 end;

 max_harm_dat:=0;
 max_harm_pat:=0;
 old_max_harm:=0;


 for n:=0 to frames-1 do //       
 begin
  for k:=DFT_kmax-1 downto 1 do
   if (dat.MaxHarm[n][k] > 0)then
      begin
        old_max_harm:=k;
        break;
      end;
  if old_max_harm > max_harm_dat then max_harm_dat:=old_max_harm;
 end;//for

 sum_all:=0;
 sum_frame:=0;
 np:=0;
 for n:=start_posX_dat to stop_posX_dat do
 begin
   for k:=1 to max_harm_dat do
   begin
    if Dat.MaxHarm[n][k] > 0 then
     begin
        ddd:=Dat.Data[n][k];
        ppp:=Pat.Data[start_posX_pat+np][k];
        if Pat.MaxHarm[start_posX_pat+np][k] > 0 then inc(sum_frame);
     end
   end;
   inc(np);
 end;
 pr:=sum_frame;


end;


procedure TImageSound.CreateDataNext;
var k,n,frames:integer;
begin
 frames:=Length(Start_dim);

 for n:=0 to frames - 1 do
 begin
   for k:=0 to DFT_kmax - 1 do
   begin
     if n=0 then
      begin
       DataUp[n][k]:=0;
       DataRight[n][k]:=0;
       DataDown[n][k]:=0;
      end
      else
      begin
       if k < DFT_kmax - 2 then DataUp[n][k]:=Data[n][k] - Data[n-1][k+1]
                           else DataUp[n][k]:=Data[n][k];
       DataRight[n][k]:=Data[n][k] - Data[n-1][k];
       if k > 0 then DataDown[n][k]:=Data[n][k] - Data[n-1][k-1]
                else DataDown[n][k]:=Data[n][k];
      end;
   end;
 end;


end;

end.
